unit DosAdventure.Remote;

interface

uses
  DosAdventure;

type
  TDosAdventureRemoteFactory = class(TDosAdventureFactory)
  protected
    function InternalCreateDosAdventure: TDosAdventure; override;
  end;

implementation

uses
  ClientModuleUnit1, System.Threading, System.Classes, System.SysUtils;

type
  TDosAdventureRemote = class(TDosAdventure)
  private type
    TTaskProcedure = reference to procedure(out AResponse: string; out ARunning: Boolean);
  private
    FClientModule: TClientModule1;
    FIsRunning: Boolean;
    procedure RunTask(AProc: TTaskProcedure);
  protected
    property ClientModule: TClientModule1 read FClientModule;
  public
    constructor Create;
    destructor Destroy; override;
    procedure SendText(const AText: string); override;
    procedure Start; override;
    function IsRunning: Boolean; override;
  end;

{ TDosAdventureRemoteFactory }

function TDosAdventureRemoteFactory.InternalCreateDosAdventure: TDosAdventure;
begin
  result := TDosAdventureRemote.Create;
end;

constructor TDosAdventureRemote.Create;
begin
  inherited Create;
  FClientModule := TClientModule1.Create(nil);
end;

destructor TDosAdventureRemote.Destroy;
begin
  FClientModule.Free;
  inherited Destroy;
end;

function TDosAdventureRemote.IsRunning: Boolean;
begin
  result := FIsRunning;
end;

procedure TDosAdventureRemote.RunTask(AProc: TTaskProcedure);
begin
  { Der Aufruf erfolgt aus Hauptthread heraus. So ein REST-Aufruf kann aber auch schon mal
    lnger dauern - insbesondere wenn er fehlschlgt. Daher verlagern wir die eigentliche
    Kommunikation mit dem Server in eine Task. }
  TTask.Run(
    procedure
    var
      response: string;
      running: Boolean;
    begin
      try
        AProc(response, running);
      except
        { Exceptions in einem Thread behandelt man besser selbst! }
        on E:Exception do begin
          response := E.Message;
          running := false;
        end;
      end;
      { Da wir hier in einem eigenen Thread laufen, mssen wir das Ergebnis wieder
        mit dem Hauptthread synchronisieren }
      TThread.Queue(nil,
        procedure
        begin
          FIsRunning := running;
          NotifyResponse(response);
        end);
    end);
end;

procedure TDosAdventureRemote.SendText(const AText: string);
//var
//  response: string;
begin
//  ClientModule.SendText(AText, response, FIsRunning);
//  NotifyResponse(response);

  RunTask(
    procedure(out AResponse: string; out ARunning: Boolean)
    begin
      ClientModule.SendText(AText, AResponse, ARunning);
    end);
end;

procedure TDosAdventureRemote.Start;
//var
//  response: string;
begin
//  ClientModule.Start(response, FIsRunning);
//  NotifyResponse(response);

  RunTask(
    procedure(out AResponse: string; out ARunning: Boolean)
    begin
      ClientModule.Start(AResponse, ARunning);
    end);
end;

end.
