unit DosAdventure;

interface

type
  TNotifyResponseEvent = procedure(Sender: TObject; const AResponse: string) of object;

  TDosAdventure = class
  private
    FOnNotifyResponse: TNotifyResponseEvent;
  protected
    procedure NotifyResponse(const AResponse: string);
  public
    function IsRunning: Boolean; virtual;
    procedure SendText(const AText: string); virtual;
    procedure Start; virtual;
    property OnNotifyResponse: TNotifyResponseEvent read FOnNotifyResponse write FOnNotifyResponse;
  end;

  TDosAdventureFactory = class
  private
    class var FInstance: TDosAdventureFactory;
    class destructor Destroy;
  protected
    function InternalCreateDosAdventure: TDosAdventure; virtual; abstract;
  public
    class function CreateDosAdventure: TDosAdventure;
    class property Instance: TDosAdventureFactory read FInstance write FInstance;
  end;

implementation

function TDosAdventure.IsRunning: Boolean;
begin
  Result := false;
end;

procedure TDosAdventure.NotifyResponse(const AResponse: string);
begin
  if Assigned(FOnNotifyResponse) then
    FOnNotifyResponse(Self, AResponse);
end;

procedure TDosAdventure.SendText(const AText: string);
begin
end;

procedure TDosAdventure.Start;
begin
end;

class destructor TDosAdventureFactory.Destroy;
begin
  FInstance.Free;
end;

class function TDosAdventureFactory.CreateDosAdventure: TDosAdventure;
begin
  if Instance <> nil then begin
    Result := Instance.InternalCreateDosAdventure;
  end
  else begin
    Result := TDosAdventure.Create;
  end;
end;

end.
